# Basic Concepts

Some of the used terminology may not be usual. To avoid confusion,
the following defines the basic concepts 

<dl>
    <dt id="#experiment">experiment</dt>
    <dd>
      <p>A combination of <a href="#suite">benchmark suites</a> and
      <a href="#executor">executors</a>.</p>
      
      <p>ReBench executes experiments to collect the desired measurements.</p>
    </dd>

    <dt id="suite">benchmark suite</dt>
    <dd>
      A set of <a href="#benchmark">benchmarks</a>
      which is used to define <a href="#experiment">experiments</a>.
    </dd>
    
    <dt id="executor">executor</dt>
    <dd>
    <p>A named set of settings for the executor of a
       <a href="#suite">benchmark suite</a>.</p>
    <p>Typically, this is one specific executable with a set of
      startup parameters. The executable will be used to execute
      <a href="#benchmark">benchmarks</a> from a <a href="#suite">suite</a>.</p>
    </dd>
    
    <dt id="benchmark">benchmark</dt>
    <dd>
      <p>A program to be executed by an <a href="#executor">executor</a>.</p>
      
      <p>A benchmark can define a number of different <a href="#variable">variables</a>
      that can be varied, for instance, to change the input data set,
      the number of cores to be used, etc.</p>
    </dd>
    
    <dt id="variable">variable</dt>
    <dd>
        <p>A dimension of the <a href="#benchmark">benchmark</a>
           that can be varied to influnce execution characteristics</p>
        <p>Currently, we have the notion of input sizes, cores, and other
           variable values. Each of them is varied independently and can potentially
           be used to enumerate a large number of <a href="#run">runs</a>.</p>
    </dd>

    <dt id="run">run</dt>
    <dd>
      <p>A concrete execution of a <a href="#benchmark">benchmark</a> by
      a specific <a href="#executor">executor</a>.</p>
      
      <p>A run is a specific combination of variables.
         It can be executed multiple times. Each time is referred to as an
         <a href="#invocation">invocation</a>.
         One run itself can also execute a benchmark multiple times, which
         we refer to as <a href="#iteration">iterations</a> of a run.</p>
      <p>One run can generate multiple <a href="#data-point">data points</a>.</p>
    </dd>
    
    <dt id="invocation">invocation</dt>
    <dd>
        The execution of a <a href="#run">run</a>. It may execute multiple
        <a href="#iteration">iterations</a> of a <a href="#benchmark">benchmark</a>.
    </dd>
    
    <dt id="iteration">iteration</dt>
    <dd>
        The execution of a benchmark within an executor
        <a href="#invocation">invocation</a>.
        An iteration is expected to generate one
        <a href="#data-point">data point</a>, possibly including
        multiple <a href="#measurement">measurements</a>.
    </dd>

    <dt id="data-point">data point</dt>
    <dd>
      A set of <a href="#measurement">measurements</a> generated by
      an <a href="#iteration">iteration</a>.
    </dd>
    
    <dt id="measurement">measurement</dt>
    <dd>One value for one specific criterion.</dd>
</dl>
